@@ -1,4 +1,3 @@
-# -*- coding=utf-8 -*-
 # |~~\'    |~~
 # |__/||~~\|--|/~\\  /
 # |   ||__/|__|   |\/
@@ -8,7 +7,7 @@ import os
 import sys
 import warnings
 
-from .__version__ import __version__    # noqa
+from pipenv.__version__ import __version__    # noqa
 
 
 PIPENV_ROOT = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
@@ -26,7 +25,8 @@ warnings.filterwarnings("ignore", category=DependencyWarning)
 warnings.filterwarnings("ignore", category=ResourceWarning)
 warnings.filterwarnings("ignore", category=UserWarning)
 
-
+# Load patched pip instead of system pip
+os.environ["PIP_SHIMS_BASE_MODULE"] = fs_str("pipenv.patched.notpip")
 os.environ["PIP_DISABLE_PIP_VERSION_CHECK"] = fs_str("1")
 
 # Hack to make things work better.
