@@ -2,32 +2,23 @@
 Package containing all pip commands
 """
 
-# The following comment should be removed at some point in the future.
-# mypy: disallow-untyped-defs=False
-
-from __future__ import absolute_import
-
 import importlib
 from collections import OrderedDict, namedtuple
+from typing import Any, Dict, Optional
 
-from pipenv.patched.notpip._internal.utils.typing import MYPY_CHECK_RUNNING
-
-if MYPY_CHECK_RUNNING:
-    from typing import Any
-    from pipenv.patched.notpip._internal.cli.base_command import Command
-
+from pipenv.patched.notpip._internal.cli.base_command import Command
 
 CommandInfo = namedtuple('CommandInfo', 'module_path, class_name, summary')
 
 # The ordering matters for help display.
 #    Also, even though the module path starts with the same
-# "pipenv.patched.notpip._internal.commands" prefix in each case, we include the full path
+# "pip._internal.commands" prefix in each case, we include the full path
 # because it makes testing easier (specifically when modifying commands_dict
 # in test setup / teardown by adding info for a FakeCommand class defined
 # in a test-related module).
 #    Finally, we need to pass an iterable of pairs here rather than a dict
 # so that the ordering won't be lost when using Python 2.7.
-commands_dict = OrderedDict([
+commands_dict: Dict[str, CommandInfo] = OrderedDict([
     ('install', CommandInfo(
         'pipenv.patched.notpip._internal.commands.install', 'InstallCommand',
         'Install packages.',
@@ -64,6 +55,14 @@ commands_dict = OrderedDict([
         'pipenv.patched.notpip._internal.commands.search', 'SearchCommand',
         'Search PyPI for packages.',
     )),
+    ('cache', CommandInfo(
+        'pipenv.patched.notpip._internal.commands.cache', 'CacheCommand',
+        "Inspect and manage pip's wheel cache.",
+    )),
+    ('index', CommandInfo(
+        'pipenv.patched.notpip._internal.commands.index', 'IndexCommand',
+        "Inspect information available from package indexes.",
+    )),
     ('wheel', CommandInfo(
         'pipenv.patched.notpip._internal.commands.wheel', 'WheelCommand',
         'Build wheels from your requirements.',
@@ -84,11 +83,10 @@ commands_dict = OrderedDict([
         'pipenv.patched.notpip._internal.commands.help', 'HelpCommand',
         'Show help for commands.',
     )),
-])  # type: OrderedDict[str, CommandInfo]
+])
 
 
-def create_command(name, **kwargs):
-    # type: (str, **Any) -> Command
+def create_command(name: str, **kwargs: Any) -> Command:
     """
     Create an instance of the Command class with the given name.
     """
@@ -100,7 +98,7 @@ def create_command(name, **kwargs):
     return command
 
 
-def get_similar_commands(name):
+def get_similar_commands(name: str) -> Optional[str]:
     """Command name auto-correct."""
     from difflib import get_close_matches
 
@@ -111,4 +109,4 @@ def get_similar_commands(name):
     if close_commands:
         return close_commands[0]
     else:
-        return False
+        return None
