@@ -58,9 +58,10 @@ if DEBUNDLED:
     sys.path[:] = glob.glob(os.path.join(WHEEL_DIR, "*.whl")) + sys.path
 
     # Actually alias all of our vendored dependencies.
+    vendored("appdirs")
     vendored("cachecontrol")
+    vendored("certifi")
     vendored("colorama")
-    vendored("contextlib2")
     vendored("distlib")
     vendored("distro")
     vendored("html5lib")
@@ -74,8 +75,6 @@ if DEBUNDLED:
     vendored("pep517")
     vendored("pkg_resources")
     vendored("progress")
-    vendored("pytoml")
-    vendored("retrying")
     vendored("requests")
     vendored("requests.exceptions")
     vendored("requests.packages")
@@ -106,4 +105,7 @@ if DEBUNDLED:
     vendored("requests.packages.urllib3.util.ssl_")
     vendored("requests.packages.urllib3.util.timeout")
     vendored("requests.packages.urllib3.util.url")
+    vendored("resolvelib")
+    vendored("tenacity")
+    vendored("tomli")
     vendored("urllib3")
